package cn.rankin.apiweb.service.user;

import cn.rankin.apiweb.code.ApiWebCode;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.data.api.user.dto.TerminalDeviceDTO;
import cn.rankin.data.api.user.entity.UserRecommend;
import cn.rankin.data.api.user.entity.UserTag;
import cn.rankin.data.api.user.vo.TerminalDeviceVo;
import cn.rankin.data.api.user.vo.TerminalUserVo;
import cn.rankin.data.api.user.vo.WhiteUserVo;
import org.springframework.cloud.netflix.feign.FeignClient;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@FeignClient(name = "${service.user.name}", fallback = UserClient.UserServiceHystrix.class)
public interface UserClient {

    @RequestMapping(value = "/user/code/{code}", method = RequestMethod.GET)
    APIResult<TerminalUserVo> loadUserByEid(@PathVariable("code") String code);

    @RequestMapping(value = "/user/{id}", method = RequestMethod.GET)
    APIResult<TerminalUserVo> findUserById(@PathVariable("id") String id);

    @RequestMapping(value = "/device/bind", method = RequestMethod.POST)
    APIResult<TerminalDeviceVo> deviceBind(@RequestBody TerminalDeviceDTO terminalDeviceDTO);

    @RequestMapping(value = "/device/unbind", method = RequestMethod.DELETE)
    APIResult<Boolean> deviceUnbind(@RequestParam("userId") String userId);

    @RequestMapping(value = "/device/uid/{uid}", method = RequestMethod.GET)
    APIResult<TerminalDeviceVo> findDeviceByUid(@PathVariable("userId") String uid);

    @RequestMapping(value = "/device/deviceCode/{deviceCode}", method = RequestMethod.GET)
    APIResult<TerminalDeviceVo> findByDeviceCode(@PathVariable("deviceCode") String deviceCode);

    @RequestMapping(value = "/white/user/{userId}", method = RequestMethod.GET)
    APIResult<WhiteUserVo> findWhiteUserById(@PathVariable("userId") String userId);

    @RequestMapping(value = "/userRecommend/courses/uid/{uid}", method = RequestMethod.GET)
    List<UserRecommend> getUserRecommendCourses(@RequestParam("uid") String uid);

    @RequestMapping(value = "/userTag/code/{code}", method = RequestMethod.GET)
    List<UserTag> findUserTagByCodeUid(@PathVariable("code") String code, @RequestParam("uid") String uid);


    @Component
    class UserServiceHystrix implements UserClient {

        @Override
        public APIResult loadUserByEid(String code) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }

        @Override
        public APIResult deviceBind(TerminalDeviceDTO terminalDeviceDTO) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }

        @Override
        public APIResult deviceUnbind(String userId) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }

        @Override
        public APIResult findDeviceByUid(String uid) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }

        @Override
        public APIResult findUserById(String id) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }

        @Override
        public APIResult findByDeviceCode(String deviceCode) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }

        @Override
        public APIResult<WhiteUserVo> findWhiteUserById(String userId) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }

        @Override
        public List<UserRecommend> getUserRecommendCourses(String uid) {
            return null;
        }

        @Override
        public List<UserTag> findUserTagByCodeUid(String code, String uid) {
            return null;
        }

    }
}
