package cn.rankin.userservice.service;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.data.api.user.dto.TerminalDeviceDTO;
import cn.rankin.data.api.user.entity.TerminalDevice;
import cn.rankin.userservice.code.UserServiceAPICode;
import cn.rankin.userservice.repository.TerminalDeviceRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;

@Service
public class TerminalDeviceService {

    @Autowired
    private TerminalDeviceBindLogService terminalDeviceBindLogService;

    @Autowired
    private TerminalDeviceRepository terminalDeviceRepository;

    @Transactional
    public APIResult<TerminalDevice> bind(TerminalDeviceDTO terminalDeviceDTO) {
        String deviceCode = terminalDeviceDTO.getDeviceCode();
        String userId = terminalDeviceDTO.getUserId();
        String brand = terminalDeviceDTO.getBrand();
        String ip = terminalDeviceDTO.getIp();
        String modelNo = terminalDeviceDTO.getModelNo();
        String terminal = terminalDeviceDTO.getTerminal();

        TerminalDevice terminalDevice = terminalDeviceRepository.findByDeviceCodeOrUserId(deviceCode, userId);
        if (terminalDevice != null) {
            if (userId.equals(terminalDevice.getUserId())) {
                return APIResult.error(UserServiceAPICode.USER_IS_BOUND);
            }else {
                return APIResult.error(UserServiceAPICode.DEVICE_IS_BOUND);
            }
        }

        terminalDevice = new TerminalDevice();
        terminalDevice.setUserId(userId);
        terminalDevice.setDeviceCode(deviceCode);
        terminalDevice.setBrand(brand);
        terminalDevice.setModelNo(modelNo);
        terminalDevice.setIp(ip);
        terminalDevice.setTerminal(terminal);

        TerminalDevice deviceBind = terminalDeviceRepository.save(terminalDevice);
        terminalDeviceBindLogService.insert(deviceCode, userId, brand, ip, modelNo, terminal);

        return APIResult.ok(deviceBind);
    }

    @Transactional
    public APIResult<Boolean> unbind(String userId) {
        TerminalDevice terminalDevice = terminalDeviceRepository.findByUserId(userId);
        if (terminalDevice == null) {
            return APIResult.error(UserServiceAPICode.NOT_EXISTS);
        }
        terminalDeviceRepository.deleteByUserId(userId);
        return APIResult.ok(Boolean.TRUE);
    }

/*    public APIResult<TerminalDevice> findByUserId(String userId) {
        TerminalDevice terminalDevice = terminalDeviceRepository.findByUserId(userId);
        if (terminalDevice == null) {
            return APIResult.error(UserServiceAPICode.NOT_EXISTS);
        }
        return APIResult.ok(terminalDevice);
    }*/

    public TerminalDevice findByUserId(String userId) {
        return terminalDeviceRepository.findByUserId(userId);
    }

    public APIResult<TerminalDevice> findByDeviceCode(String deviceCode) {
        TerminalDevice terminalDevice = terminalDeviceRepository.findByDeviceCode(deviceCode);
        if (terminalDevice == null) {
            return APIResult.error(UserServiceAPICode.NOT_EXISTS);
        }
        return APIResult.ok(terminalDevice);
    }
}
